/***************************************************************************
                          texttospeechconfigurationdialog.cpp  -  description
                             -------------------
    begin                : Son Sep 8 2002
    copyright            : (C) 2002 by Gunnar Schmi Dt
    email                : kmouth@schmi-dt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Id: texttospeechconfigurationdialog.cpp,v 1.4.2.1 2002/11/04 15:36:37 gunnars Exp $

#include "texttospeechconfigurationdialog.h"
#include <kconfig.h>
#include <qtextcodec.h>
#include <qptrlist.h>
#include <qlayout.h>
#include <qwhatsthis.h>
#include <kcombobox.h>
#include <klocale.h>
#include <qlabel.h>
#include "speech.h"

TextToSpeechConfigurationDialog::TextToSpeechConfigurationDialog (QWidget *parent)
   : KDialogBase(parent, "Text-to-speech configuration", false,
                 i18n("Text-to-speech configuration"), Ok|Apply|Cancel, Ok, true)
{
   ttsSystem = new TextToSpeechSystem();

   QWidget *page = new QWidget(this);
   setMainWidget(page);
   QVBoxLayout *topLayout = new QVBoxLayout (page, 0, spacingHint());

   QLabel *label = new QLabel (i18n("Command for speaking texts:"), page);
   topLayout->addWidget (label);

   urlReq = new KURLRequester (page);
   urlReq->setShowLocalProtocol (false);
   QWhatsThis::add (urlReq, i18n("This field specifies both the command KMouth uses for speaking texts and its parameters. If you want to pass the text as a parameter, write %t at the place where the text should be inserted."));
   topLayout->addWidget (urlReq);
   
   QLabel *label2 = new QLabel (i18n("Character encoding:"), page);
   topLayout->addWidget (label2);

   characterCodingBox = new KComboBox (page, "characterCoding");
   QWhatsThis::add (characterCodingBox, i18n("This combo box specifies which character encoding is used for passing the text."));
   topLayout->addWidget (characterCodingBox);
   buildCodecList();

   stdInButton = new QCheckBox (i18n("Send the data as standard input"), page);
   QWhatsThis::add (stdInButton, i18n("This check box specifies whether the text is sent as standard input to the speech synthesizer."));
   topLayout->addWidget(stdInButton);
   
   topLayout->addStretch(10);
}

TextToSpeechConfigurationDialog::~TextToSpeechConfigurationDialog() {
}

void TextToSpeechConfigurationDialog::buildCodecList () {
   QString local = i18n("Local")+" (";
   local += QTextCodec::codecForLocale()->name();
   local += ")";
   characterCodingBox->insertItem (local, Speech::Local);
   characterCodingBox->insertItem (i18n("Latin1"), Speech::Latin1);
   characterCodingBox->insertItem (i18n("Unicode"), Speech::Unicode);
   for (int i = 0; i < ttsSystem->codecList->count(); i++ )
      characterCodingBox->insertItem (ttsSystem->codecList->at(i)->name(), Speech::UseCodec + i);
}


void TextToSpeechConfigurationDialog::slotCancel() {
  KDialogBase::slotCancel();
  urlReq->setURL (ttsSystem->ttsCommand);
  stdInButton->setChecked (ttsSystem->stdIn);
  characterCodingBox->setCurrentItem(ttsSystem->codec);
}

void TextToSpeechConfigurationDialog::slotOk() {
  KDialogBase::slotOk();
  ttsSystem->ttsCommand = urlReq->url();
  ttsSystem->stdIn = stdInButton->isChecked();
  ttsSystem->codec = characterCodingBox->currentItem();
}

void TextToSpeechConfigurationDialog::slotApply() {
  KDialogBase::slotApply();
  ttsSystem->ttsCommand = urlReq->url();
  ttsSystem->stdIn = stdInButton->isChecked();
  ttsSystem->codec = characterCodingBox->currentItem();
}

TextToSpeechSystem *TextToSpeechConfigurationDialog::getTTSSystem() const {
   return ttsSystem;
}

void TextToSpeechConfigurationDialog::readOptions (KConfig *config) {
  config->setGroup("TTS System");
  ttsSystem->ttsCommand = config->readEntry("Command", "cat -");
  ttsSystem->stdIn = config->readBoolEntry("StdIn", true);
  urlReq->setURL (ttsSystem->ttsCommand);
  stdInButton->setChecked (ttsSystem->stdIn);
  QString codec = config->readEntry("Codec", "Local");
  if (codec == "Local")
     ttsSystem->codec = Speech::Local;
  else if (codec == "Latin1")
     ttsSystem->codec = Speech::Latin1;
  else if (codec == "Unicode")
     ttsSystem->codec = Speech::Unicode;
  else {
     ttsSystem->codec = Speech::Local;
     for (int i = 0; i < ttsSystem->codecList->count(); i++ )
        if (codec == ttsSystem->codecList->at(i)->name())
           ttsSystem->codec = Speech::UseCodec + i;
  }
  characterCodingBox->setCurrentItem(ttsSystem->codec);
}

void TextToSpeechConfigurationDialog::saveOptions (KConfig *config) {
  config->setGroup("TTS System");
  config->writeEntry("Command", ttsSystem->ttsCommand);
  config->writeEntry("StdIn", ttsSystem->stdIn);
  if (ttsSystem->codec == Speech::Local)
     config->writeEntry("Codec", "Local");
  else if (ttsSystem->codec == Speech::Latin1)
     config->writeEntry("Codec", "Latin1");
  else if (ttsSystem->codec == Speech::Unicode)
     config->writeEntry("Codec", "Unicode");
  else config->writeEntry("Codec",
         ttsSystem->codecList->at (ttsSystem->codec-Speech::UseCodec)->name());
  
}

/*
 * $Log: texttospeechconfigurationdialog.cpp,v $
 * Revision 1.4.2.1  2002/11/04 15:36:37  gunnars
 * combo box for character encoding added
 *
 * Revision 1.4  2002/10/07 17:09:33  gunnars
 * What's this? texts added
 *
 * Revision 1.3  2002/10/02 14:55:33  gunnars
 * Fixed Speak-empty-phrase-crash bug and added some i18n() encodings
 *
 * Revision 1.2  2002/09/08 19:29:42  gunnar
 * Configuration dialog improved
 *
 * Revision 1.1  2002/09/08 17:12:55  gunnar
 * Configuration dialog added
 *
 */
